var authBool = false;
var BASE_URL = window.location.origin;
// default global variables
var JAYA_URL = 'https://devel.hla.triumf.ca';
var TUNEX_URL = 'https://tunex.dev.kube.dmz.triumf.ca';
var MCAT_URL = 'https://mcat.triumf.ca';
var SNAPSHOT_URL = 'https://devel.hla.triumf.ca/snapshot';
var ENVELOPE_URL = 'https://dev.envelope.triumf.ca';
    
function update_URLs() {
  if ( BASE_URL.includes("localhost:4994") || BASE_URL.includes("0.0.0.0:4994") || BASE_URL.includes("devel.hla.triumf.ca") || BASE_URL.includes("dev.kube.dmz.triumf.ca")) {
    // Running localhost on port 4994 is flag that developer wants to interact
    //  with the devel jaya for testing read/write without affecting epics
    JAYA_URL = 'https://devel.hla.triumf.ca';
    TUNEX_URL = 'https://tunex.dev.kube.dmz.triumf.ca';
    SNAPSHOT_URL = 'https://devel.hla.triumf.ca/snapshot';

    $('#jaya-link').attr("href", JAYA_URL+"/jaya");
    $('#tunex-link').attr("href", TUNEX_URL);
    $('#mcat-link').attr("href", MCAT_URL);
    $('#snapshot-link').attr("href", SNAPSHOT_URL);
    $('#envelope-link').attr("href", ENVELOPE_URL);

    jaya_devMode();
    console.log("WARNING - jaya.js is running in development mode!!!")
    alert("WARNING - jaya.js is running in development mode!!! reading/writing to PVs has no connection to EPICS. PVs read from jaya DO NOT REPRESENT actual control system.")
  } 

  // beta, prod, => let these guys pass url as is so that the vpn.beta.hla.triumf.ca proxy still works.
  else if ( BASE_URL.includes("hla.triumf.ca") ) {
    JAYA_URL = BASE_URL; 
    TUNEX_URL = 'https://tunex.triumf.ca';
    SNAPSHOT_URL = 'https://beta.hla.triumf.ca/snapshot';

    $('#jaya-link').attr("href", JAYA_URL+"/jaya");
    $('#tunex-link').attr("href", TUNEX_URL);
    $('#mcat-link').attr("href", MCAT_URL);
    $('#snapshot-link').attr("href", SNAPSHOT_URL);
    $('#envelope-link').attr("href", ENVELOPE_URL);
  }

  // kube prod
  else if (  BASE_URL.includes("tunex.triumf.ca") || BASE_URL.includes("dev.envelope.triumf.ca")) {
    JAYA_URL = 'https://beta.hla.triumf.ca'; 
    TUNEX_URL = 'https://tunex.triumf.ca';
    SNAPSHOT_URL = 'https://beta.hla.triumf.ca/snapshot';

    $('#jaya-link').attr("href", JAYA_URL+"/jaya");
    $('#tunex-link').attr("href", TUNEX_URL);
    $('#mcat-link').attr("href", MCAT_URL);
    $('#snapshot-link').attr("href", SNAPSHOT_URL);
    $('#envelope-link').attr("href", ENVELOPE_URL);
  }

  else {
    // default jaya location mainly for running locally when want real readbacks 
    // (have to run on port other 4994.) 
    JAYA_URL = 'https://beta.hla.triumf.ca'; 
    TUNEX_URL = 'https://tunex.triumf.ca';
    SNAPSHOT_URL = 'https://beta.hla.triumf.ca/snapshot';

    $('#jaya-link').attr("href", JAYA_URL+"/jaya");
    $('#tunex-link').attr("href", TUNEX_URL);
    $('#mcat-link').attr("href", MCAT_URL);
    $('#snapshot-link').attr("href", SNAPSHOT_URL);
    $('#envelope-link').attr("href", ENVELOPE_URL);
  }
}


function jaya_devMode() {
  var txt = "<div class='alert alert-warning alert-dismissible'>"
  txt += "WARNING - app is running in development mode!!! "
  txt += "reading/writing to PVs has no connection to EPICS. "
  txt += "PV values DO NOT REPRESENT the current state of the control system.</div>";
  $('#dev_mode_warning').html(txt);
}

function jaya_checkAuth(script_root) {
  var myurl = JAYA_URL +  "/jaya/checkAuth";
  return $.ajax({
        url: myurl,
        type: "GET",
        dataType: "json",
        success: function(response) {
          jaya_updateAuth(response);
        },
        error: function(xhr, status, error) {
            console.log({
                message: "Error in apex/static/js/jaya.js function jaya_checkAuth()"
            });
        },
  });   
}


function jaya_updateAuth(response) {
  authBool = false;
  //console.log(response)
  var facilities = ['isac','elinac','cyclotron','teststand','magnettest']
  var fac_with_both = [];
  var fac_with_only_token = [];

  for (var i = 0; i < facilities.length; i++) { 
    if ( response['has_token_'+ facilities[i]] ) {
      if ( response['hlaepics_access_'+ facilities[i]] ) {
        fac_with_both.push(facilities[i]);
      }
      else {
        fac_with_only_token.push(facilities[i]);
      }
    }
  }

  // at least one facility has token AND epics access
  if ( fac_with_both.length != 0 ) {
    $('#tokenAuth').removeClass('badge-secondary').addClass('badge-success');
    $('#epicsAuth').removeClass('badge-secondary').addClass('badge-success');
    var output = ''
    for (i = 0; i < fac_with_both.length; i++)
      output += fac_with_both[i] + " "
    $('#facilityAuth').text(output)
    authBool = true;
  }

  // at least one facility has token but none have epics access
  else if ( fac_with_only_token.length != 0 ) {
    $('#tokenAuth').removeClass('badge-secondary').addClass('badge-success');
    $('#epicsAuth').removeClass('badge-success').addClass('badge-secondary');
    var output = ''
    for (i = 0; i < fac_with_only_token.length; i++)
      output += fac_with_only_token[i] + " "
    $('#facilityAuth').text(output)
  }

  // User doesn't have authorization for anything.
  else {
    $('#tokenAuth').removeClass('badge-success').addClass('badge-secondary');
    $('#epicsAuth').removeClass('badge-success').addClass('badge-secondary');
    $('#facilityAuth').text('')
  }

}


function jaya_get(submit_data, callback, error_callback) {
  var myurl = JAYA_URL + '/jaya/get';
  return $.ajax({
        url: myurl,
        data: JSON.stringify(submit_data),
        type: "POST",
        dataType: "json",
        contentType: 'application/json',
        success: function(response) {
          callback(response);
        },
        error: function(xhr, status, error) {
            var error = 'Unknown error in jaya. (likely CORS)';
            if (typeof xhr.responseJSON !== 'undefined') {
              if ('status' in xhr.responseJSON) {
                //console.log(xhr.responseJSON['status']);
                error = xhr.responseJSON['status'];
              }
              else if (typeof status !== 'undefined') {
                error = 'Unkown error of type ' + status
              }
            }
            console.log("Error in apex/static/js/jaya.js function jaya_get()");
            console.log(error);
            error_callback(error);
        },
    });   
}


function jaya_set(submit_data, app_data, callback, error_callback) {
  submit_data["app_data"] = app_data;
  var myurl = JAYA_URL + '/jaya/set';
  return $.ajax({
        url: myurl,
        data: JSON.stringify(submit_data),
        type: "POST",
        dataType: "json",
        contentType: 'application/json',
        success: function(response) {
          callback(response);
        },
        error: function(xhr, status, error) {
            var error = 'Unknown error in jaya. (likely CORS)';
            if (typeof xhr.responseJSON !== 'undefined') {
              if ('status' in xhr.responseJSON) {
                //console.log(xhr.responseJSON['status']);
                error = xhr.responseJSON['status'];
              }
              else if (typeof status !== 'undefined') {
                error = 'Unkown error of type ' + status
              }
            }
            console.log("Error in apex/static/js/jaya.js function jaya_set()");
            console.log(error);
            error_callback(error);
        },
    });   
}


function jaya_ramp(submit_data, app_data, callback, error_callback) {
  submit_data["app_data"] = app_data;
  var myurl = JAYA_URL + '/jaya/ramp';
  return $.ajax({
        url: myurl,
        data: JSON.stringify(submit_data),
        type: "POST",
        dataType: "json",
        contentType: 'application/json',
        success: function(data, status, request) {
            status_url = JAYA_URL + data.Location;
            callback(status_url);
        },
        error: function(xhr, status, error) {
            var error = 'Unknown error in jaya. (likely CORS)';
            if (typeof xhr.responseJSON !== 'undefined') {
              if ('status' in xhr.responseJSON) {
                //console.log(xhr.responseJSON['status']);
                error = xhr.responseJSON['status'];
              }
              else if (typeof status !== 'undefined') {
                error = 'Unkown error of type ' + status
              }
            }
            console.log("Error in apex/static/js/jaya.js function jaya_ramp()");
            console.log(error);
            error_callback(error);
        },
    });   
}


function jaya_measure(measure_array, app_data, callback, error_callback) {
  var submit_data = {}
  submit_data["measure"] = measure_array;
  submit_data["app_data"] = app_data;
  // example: 
  // app_data = {'app_name': 'snapshot', 'measure_header': MEASURE_HEADER, 'measure_id': MEASURE_ID}
  var myurl = JAYA_URL + '/jaya/measure';
  return $.ajax({
        url: myurl,
        data: JSON.stringify(submit_data),
        type: "POST",
        dataType: "json",
        contentType: 'application/json',
        success: function(data, status, request) {
            status_url = JAYA_URL + data.Location;
            callback(status_url);
        },
        error: function(xhr, status, error) {
            var error = 'Unknown error in jaya. (likely CORS)';
            if (typeof xhr.responseJSON !== 'undefined') {
              if ('status' in xhr.responseJSON) {
                //console.log(xhr.responseJSON['status']);
                error = xhr.responseJSON['status'];
              }
              else if (typeof status !== 'undefined') {
                error = 'Unkown error of type ' + status
              }
            }
            console.log("Error in apex/static/js/jaya.js function jaya_measure()");
            console.log(error);
            error_callback(error);
        },
    });   
}


function jaya_stop(pv_name, callback, error_callback) {
  var myurl = JAYA_URL + '/jaya/stop/' + pv_name;
  return $.ajax({
        url: myurl,
        type: "GET",
        dataType: "json",
        success: function(response) {
            var success_message = response['status']
            callback(success_message);
        },
        error: function(xhr, status, error) {
            var error = 'Unknown error in jaya. (likely CORS)';
            if (typeof xhr.responseJSON !== 'undefined') {
              if ('status' in xhr.responseJSON) {
                //console.log(xhr.responseJSON['status']);
                error = xhr.responseJSON['status'];
              }
              else if (typeof status !== 'undefined') {
                error = 'Unkown error of type ' + status
              }
            }
            console.log("Error in apex/static/js/jaya.js function jaya_stop()");
            console.log(error);
            error_callback(error);
        },
    });   
}


function create_hla_measurement(measure_header, callback=false, error_callback=false) {
  var myurl = JAYA_URL + '/jaya/create_measurement';
  var submit_data = {'measure_header': measure_header}
  return $.ajax({
        url: myurl,
        data: JSON.stringify(submit_data),
        type: "POST",
        dataType: "json",
        contentType: 'application/json',
        success: function(response) {
            console.log("HLA beam DB measurement created")
            console.log("MEASURE_ID = " + response['measure_id'])
            MEASURE_ID = response['measure_id']
            if (callback) {
              // Hardcode a wait to give jaya time to finish creating measurement
              setTimeout(function () {
                callback(response['measure_id']);
              }, 3000);
            }
        },
        error: function(xhr, status, error) {
            console.log("Error in apex/static/js/jaya.js function create_hla_measurement()");
            var error = '';
            if (typeof xhr.responseJSON !== 'undefined') {
              if ('status' in xhr.responseJSON) {
                //console.log(xhr.responseJSON['status']);
                error = xhr.responseJSON['status'];
              }
              else if (typeof status !== 'undefined') {
                error = 'Unkown error of type ' + status
              }
            }
            console.log(error);
            if (error_callback) {
              error_callback(error);
            }
        },
    });   
}


function preserve_hla_data(measure_id, skip_files=false, callback=false, error_callback=false) {
/*
  Required arguments
  measure_id  : string; celery task id of measurment that should be ported 
                          over to Postgres DB

  Optional arguments
  skip_files:   list; list of celery task ids that should not be included 
                          with this measurement.

  Returns
  measure_id of record in permament DB if successful.
*/
  if (!skip_files) {
    var skip_files = [];
  }
  else if (typeof skip_files == "string") {
    var skip_files = [skip_files];
  }

  var myurl = JAYA_URL + '/jaya/preserve_data';
  var submit_data = {'measure_id': measure_id, 'skip_files': skip_files}
  return $.ajax({
        url: myurl,
        data: JSON.stringify(submit_data),
        type: "POST",
        dataType: "json",
        contentType: 'application/json',
        success: function(response) {
            console.log("Measurement successfully saved.")
            console.log("Permanent MEASURE_ID = " + response['measure_id'])
            MEASURE_ID = response['measure_id'];
            if (callback) {
              callback(response['measure_id'])
            }
        },
        error: function(xhr, status, error) {
            console.log("Error in apex/static/js/jaya.js function preserve_data()");

            if (typeof xhr.responseJSON !== 'undefined') {
              if ('status' in xhr.responseJSON) {
                //console.log(xhr.responseJSON['status']);
                error = xhr.responseJSON['status'];
              }
              else if (typeof status !== 'undefined') {
                error = 'Unkown error of type ' + status
              }
            }
            console.log(error);
            if (error_callback) {
              error_callback(error);
            }
        },
    });   
}


function update_measure_info(measure_id, kwargs, callback=false, error_callback=false) {
/*
  Required arguments
  measure_id  : int; beam DB measure_id of record to be updated

  Optional kwargs
  elog_entry   : int; integer represeting number of the associated elog_entry
  post_process : json/dict; information regarding how post-processing of data 
                  should be handeled.
  comments     : str; comments about the measurement

  Returns
  message back confirming success
*/
  var myurl = JAYA_URL + '/jaya/update_measure_info';
  var submit_data = kwargs
  submit_data['measure_id'] = measure_id

  return $.ajax({
        url: myurl,
        data: JSON.stringify(submit_data),
        type: "POST",
        dataType: "json",
        contentType: 'application/json',
        success: function(response) {
            console.log("Measurement info successfully saved.")
            if (callback) {
              callback(response['status'])
            }
        },
        error: function(xhr, status, error) {
            console.log("Error in apex/static/js/jaya.js function update_measure_info()");

            if (typeof xhr.responseJSON !== 'undefined') {
              if ('status' in xhr.responseJSON) {
                //console.log(xhr.responseJSON['status']);
                error = xhr.responseJSON['status'];
              }
              else if (typeof status !== 'undefined') {
                error = 'Unkown error of type ' + status
              }
            }
            console.log(error);
            if (error_callback) {
              error_callback(error);
            }
        },
    });   
}


function get_hla_data(measure_id, callback, error_callback) {
  var myurl = JAYA_URL + '/jaya/data/get/' + measure_id + '/';
  console.log(myurl)
  return $.ajax({
        url: myurl,
        type: "GET",
        dataType: "json",
        success: function(response) {
            console.log("Successfully received data via get_hla_data")
            callback(response);
        },
        error: function(xhr, status, error) {
            console.log("Error in apex/static/js/jaya.js function get_hla_data()");

            if (typeof xhr.responseJSON !== 'undefined') {
              if ('status' in xhr.responseJSON) {
                //console.log(xhr.responseJSON['status']);
                error = xhr.responseJSON['status'];
              }
              else if (typeof status !== 'undefined') {
                error = 'Unkown error of type ' + status
              }
            }
            console.log(error);
            error_callback(error);
        },
    });   
}

