
# Installation

For local installations use pip:
```
pip install -e https://gitlab.triumf.ca/hla/apex
```

For use in projects use the following line in the requirements.txt file:
```
git+https://gitlab.triumf.ca/hla/apex.git#egg=apex
```
specific branches or commits can be pointed to as well by changing the link.

# Usage

In the file where the main Flask application is defined import the following near the beginning:
```
from apex import mod_apex
```
Make sure it is before the line that usually looks like `app = Flask(__name__)` then after that include:
```
app.register_blueprint(mod_apex)
```
now you can use the templates!

To use the templates in the routing functions return the `render_template()` function with the template name and the required dictionaries to use the templates.

## layout_header.html

Provides the application header with the application name with GitLab and theme buttons. It requires the following dictionary:
```
header_settings = {
    'title': 'The HTML page title',
    'app_name': 'The application name',
    'app_url': 'https://what_happens_when_you_click_the_app_name',
    'gitlab_url': 'https://link_to_your_gitlab_project'
}
```
## 404.html

Extends layout_header.html, so it requires the same dictionary

## layout_navbar.html

Extends layout_header.html so it requires its dictionary as well as the following list:
```
navbar_settings = [
    {'type':'text', 'text':'This will show up as text in the navbar'},
    {'type':'link', 'text':'this is the link name', 'url':'https://link_url'},
    {'type':'dropdown', 'text':'this is the link list name', 
        'links': [
            {text':'link name', 'url':'https://link_url'},
            {text':'link name', 'url':'https://link_url'},
            {text':'link name', 'url':'https://link_url'},
            {text':'link name', 'url':'https://link_url'},
        ],
    },
]
```

Lastly, Look at `templates/macros.html` to see commonly used patterns.

## gzip Compression 
To enable gzip compression in your project install the `flask-compress` library or add it to your requirements.txt file:
```
flask-compress>=1.4.0
```
Then add the following to the top of the `__init__.py` file in the `webapp` directory:
```
from flask_compress import Compress
...
app = Flask(__name__)
app.register_blueprint(mod_apex)
Compress(app)
```